mirror of
https://gitlab.com/libeigen/eigen.git
synced 2026-04-10 11:34:33 +08:00
Resolve bad merge.
This commit is contained in:
@@ -574,6 +574,191 @@ void test_cuda_lgamma(const Scalar stddev)
|
||||
cudaFree(d_out);
|
||||
}
|
||||
|
||||
template <typename Scalar>
|
||||
void test_cuda_digamma()
|
||||
{
|
||||
Tensor<Scalar, 1> in(7);
|
||||
Tensor<Scalar, 1> out(7);
|
||||
Tensor<Scalar, 1> expected_out(7);
|
||||
out.setZero();
|
||||
|
||||
in(0) = Scalar(1);
|
||||
in(1) = Scalar(1.5);
|
||||
in(2) = Scalar(4);
|
||||
in(3) = Scalar(-10.5);
|
||||
in(4) = Scalar(10000.5);
|
||||
in(5) = Scalar(0);
|
||||
in(6) = Scalar(-1);
|
||||
|
||||
expected_out(0) = Scalar(-0.5772156649015329);
|
||||
expected_out(1) = Scalar(0.03648997397857645);
|
||||
expected_out(2) = Scalar(1.2561176684318);
|
||||
expected_out(3) = Scalar(2.398239129535781);
|
||||
expected_out(4) = Scalar(9.210340372392849);
|
||||
expected_out(5) = std::numeric_limits<Scalar>::infinity();
|
||||
expected_out(6) = std::numeric_limits<Scalar>::infinity();
|
||||
|
||||
std::size_t bytes = in.size() * sizeof(Scalar);
|
||||
|
||||
Scalar* d_in;
|
||||
Scalar* d_out;
|
||||
cudaMalloc((void**)(&d_in), bytes);
|
||||
cudaMalloc((void**)(&d_out), bytes);
|
||||
|
||||
cudaMemcpy(d_in, in.data(), bytes, cudaMemcpyHostToDevice);
|
||||
|
||||
Eigen::CudaStreamDevice stream;
|
||||
Eigen::GpuDevice gpu_device(&stream);
|
||||
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 1> > gpu_in(d_in, 7);
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 1> > gpu_out(d_out, 7);
|
||||
|
||||
gpu_out.device(gpu_device) = gpu_in.digamma();
|
||||
|
||||
assert(cudaMemcpyAsync(out.data(), d_out, bytes, cudaMemcpyDeviceToHost, gpu_device.stream()) == cudaSuccess);
|
||||
assert(cudaStreamSynchronize(gpu_device.stream()) == cudaSuccess);
|
||||
|
||||
for (int i = 0; i < 5; ++i) {
|
||||
VERIFY_IS_APPROX(out(i), expected_out(i));
|
||||
}
|
||||
for (int i = 5; i < 7; ++i) {
|
||||
VERIFY_IS_EQUAL(out(i), expected_out(i));
|
||||
}
|
||||
}
|
||||
|
||||
template <typename Scalar>
|
||||
void test_cuda_igamma()
|
||||
{
|
||||
Tensor<Scalar, 2> a(6, 6);
|
||||
Tensor<Scalar, 2> x(6, 6);
|
||||
Tensor<Scalar, 2> out(6, 6);
|
||||
out.setZero();
|
||||
|
||||
Scalar a_s[] = {Scalar(0), Scalar(1), Scalar(1.5), Scalar(4), Scalar(0.0001), Scalar(1000.5)};
|
||||
Scalar x_s[] = {Scalar(0), Scalar(1), Scalar(1.5), Scalar(4), Scalar(0.0001), Scalar(1000.5)};
|
||||
|
||||
for (int i = 0; i < 6; ++i) {
|
||||
for (int j = 0; j < 6; ++j) {
|
||||
a(i, j) = a_s[i];
|
||||
x(i, j) = x_s[j];
|
||||
}
|
||||
}
|
||||
|
||||
Scalar nan = std::numeric_limits<Scalar>::quiet_NaN();
|
||||
Scalar igamma_s[][6] = {{0.0, nan, nan, nan, nan, nan},
|
||||
{0.0, 0.6321205588285578, 0.7768698398515702,
|
||||
0.9816843611112658, 9.999500016666262e-05, 1.0},
|
||||
{0.0, 0.4275932955291202, 0.608374823728911,
|
||||
0.9539882943107686, 7.522076445089201e-07, 1.0},
|
||||
{0.0, 0.01898815687615381, 0.06564245437845008,
|
||||
0.5665298796332909, 4.166333347221828e-18, 1.0},
|
||||
{0.0, 0.9999780593618628, 0.9999899967080838,
|
||||
0.9999996219837988, 0.9991370418689945, 1.0},
|
||||
{0.0, 0.0, 0.0, 0.0, 0.0, 0.5042041932513908}};
|
||||
|
||||
|
||||
|
||||
std::size_t bytes = a.size() * sizeof(Scalar);
|
||||
|
||||
Scalar* d_a;
|
||||
Scalar* d_x;
|
||||
Scalar* d_out;
|
||||
cudaMalloc((void**)(&d_a), bytes);
|
||||
cudaMalloc((void**)(&d_x), bytes);
|
||||
cudaMalloc((void**)(&d_out), bytes);
|
||||
|
||||
cudaMemcpy(d_a, a.data(), bytes, cudaMemcpyHostToDevice);
|
||||
cudaMemcpy(d_x, x.data(), bytes, cudaMemcpyHostToDevice);
|
||||
|
||||
Eigen::CudaStreamDevice stream;
|
||||
Eigen::GpuDevice gpu_device(&stream);
|
||||
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_a(d_a, 6, 6);
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_x(d_x, 6, 6);
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_out(d_out, 6, 6);
|
||||
|
||||
gpu_out.device(gpu_device) = gpu_a.igamma(gpu_x);
|
||||
|
||||
assert(cudaMemcpyAsync(out.data(), d_out, bytes, cudaMemcpyDeviceToHost, gpu_device.stream()) == cudaSuccess);
|
||||
assert(cudaStreamSynchronize(gpu_device.stream()) == cudaSuccess);
|
||||
|
||||
for (int i = 0; i < 6; ++i) {
|
||||
for (int j = 0; j < 6; ++j) {
|
||||
if ((std::isnan)(igamma_s[i][j])) {
|
||||
VERIFY((std::isnan)(out(i, j)));
|
||||
} else {
|
||||
VERIFY_IS_APPROX(out(i, j), igamma_s[i][j]);
|
||||
}
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
template <typename Scalar>
|
||||
void test_cuda_igammac()
|
||||
{
|
||||
Tensor<Scalar, 2> a(6, 6);
|
||||
Tensor<Scalar, 2> x(6, 6);
|
||||
Tensor<Scalar, 2> out(6, 6);
|
||||
out.setZero();
|
||||
|
||||
Scalar a_s[] = {Scalar(0), Scalar(1), Scalar(1.5), Scalar(4), Scalar(0.0001), Scalar(1000.5)};
|
||||
Scalar x_s[] = {Scalar(0), Scalar(1), Scalar(1.5), Scalar(4), Scalar(0.0001), Scalar(1000.5)};
|
||||
|
||||
for (int i = 0; i < 6; ++i) {
|
||||
for (int j = 0; j < 6; ++j) {
|
||||
a(i, j) = a_s[i];
|
||||
x(i, j) = x_s[j];
|
||||
}
|
||||
}
|
||||
|
||||
Scalar nan = std::numeric_limits<Scalar>::quiet_NaN();
|
||||
Scalar igammac_s[][6] = {{nan, nan, nan, nan, nan, nan},
|
||||
{1.0, 0.36787944117144233, 0.22313016014842982,
|
||||
0.018315638888734182, 0.9999000049998333, 0.0},
|
||||
{1.0, 0.5724067044708798, 0.3916251762710878,
|
||||
0.04601170568923136, 0.9999992477923555, 0.0},
|
||||
{1.0, 0.9810118431238462, 0.9343575456215499,
|
||||
0.4334701203667089, 1.0, 0.0},
|
||||
{1.0, 2.1940638138146658e-05, 1.0003291916285e-05,
|
||||
3.7801620118431334e-07, 0.0008629581310054535,
|
||||
0.0},
|
||||
{1.0, 1.0, 1.0, 1.0, 1.0, 0.49579580674813944}};
|
||||
|
||||
std::size_t bytes = a.size() * sizeof(Scalar);
|
||||
|
||||
Scalar* d_a;
|
||||
Scalar* d_x;
|
||||
Scalar* d_out;
|
||||
cudaMalloc((void**)(&d_a), bytes);
|
||||
cudaMalloc((void**)(&d_x), bytes);
|
||||
cudaMalloc((void**)(&d_out), bytes);
|
||||
|
||||
cudaMemcpy(d_a, a.data(), bytes, cudaMemcpyHostToDevice);
|
||||
cudaMemcpy(d_x, x.data(), bytes, cudaMemcpyHostToDevice);
|
||||
|
||||
Eigen::CudaStreamDevice stream;
|
||||
Eigen::GpuDevice gpu_device(&stream);
|
||||
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_a(d_a, 6, 6);
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_x(d_x, 6, 6);
|
||||
Eigen::TensorMap<Eigen::Tensor<Scalar, 2> > gpu_out(d_out, 6, 6);
|
||||
|
||||
gpu_out.device(gpu_device) = gpu_a.igammac(gpu_x);
|
||||
|
||||
assert(cudaMemcpyAsync(out.data(), d_out, bytes, cudaMemcpyDeviceToHost, gpu_device.stream()) == cudaSuccess);
|
||||
assert(cudaStreamSynchronize(gpu_device.stream()) == cudaSuccess);
|
||||
|
||||
for (int i = 0; i < 6; ++i) {
|
||||
for (int j = 0; j < 6; ++j) {
|
||||
if ((std::isnan)(igammac_s[i][j])) {
|
||||
VERIFY((std::isnan)(out(i, j)));
|
||||
} else {
|
||||
VERIFY_IS_APPROX(out(i, j), igammac_s[i][j]);
|
||||
}
|
||||
}
|
||||
}
|
||||
}
|
||||
|
||||
template <typename Scalar>
|
||||
void test_cuda_erf(const Scalar stddev)
|
||||
{
|
||||
@@ -667,30 +852,46 @@ void test_cxx11_tensor_cuda()
|
||||
CALL_SUBTEST_3(test_cuda_convolution_2d<RowMajor>());
|
||||
CALL_SUBTEST_3(test_cuda_convolution_3d<ColMajor>());
|
||||
CALL_SUBTEST_3(test_cuda_convolution_3d<RowMajor>());
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<float>(1.0f));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<float>(100.0f));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<float>(0.01f));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<float>(0.001f));
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_digamma<float>());
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_erf<float>(1.0f));
|
||||
CALL_SUBTEST_4(test_cuda_erf<float>(100.0f));
|
||||
CALL_SUBTEST_4(test_cuda_erf<float>(0.01f));
|
||||
CALL_SUBTEST_4(test_cuda_erf<float>(0.001f));
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_erfc<float>(1.0f));
|
||||
// CALL_SUBTEST(test_cuda_erfc<float>(100.0f));
|
||||
CALL_SUBTEST_4(test_cuda_erfc<float>(5.0f)); // CUDA erfc lacks precision for large inputs
|
||||
CALL_SUBTEST_4(test_cuda_erfc<float>(0.01f));
|
||||
CALL_SUBTEST_4(test_cuda_erfc<float>(0.001f));
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<double>(1.0));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<double>(100.0));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<double>(0.01));
|
||||
CALL_SUBTEST_4(test_cuda_lgamma<double>(0.001));
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_digamma<double>());
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_erf<double>(1.0));
|
||||
CALL_SUBTEST_4(test_cuda_erf<double>(100.0));
|
||||
CALL_SUBTEST_4(test_cuda_erf<double>(0.01));
|
||||
CALL_SUBTEST_4(test_cuda_erf<double>(0.001));
|
||||
|
||||
CALL_SUBTEST_4(test_cuda_erfc<double>(1.0));
|
||||
// CALL_SUBTEST(test_cuda_erfc<double>(100.0));
|
||||
CALL_SUBTEST_4(test_cuda_erfc<double>(5.0)); // CUDA erfc lacks precision for large inputs
|
||||
CALL_SUBTEST_4(test_cuda_erfc<double>(0.01));
|
||||
CALL_SUBTEST_4(test_cuda_erfc<double>(0.001));
|
||||
|
||||
CALL_SUBTEST_5(test_cuda_igamma<float>());
|
||||
CALL_SUBTEST_5(test_cuda_igammac<float>());
|
||||
|
||||
CALL_SUBTEST_5(test_cuda_igamma<double>());
|
||||
CALL_SUBTEST_5(test_cuda_igammac<double>());
|
||||
}
|
||||
|
||||
Reference in New Issue
Block a user